@@ -19,23 +19,19 @@
 #include <config.h>
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-ftp-dialog.h"
-#include "gnome-cmd-ftp-server-funcs.h"
+#include "gnome-cmd-con-ftp.h"
 #include "gnome-cmd-data.h"
 #include "imageloader.h"
-#include "gnome-cmd-connection-funcs.h"
-#include "gnome-cmd-file-selector-funcs.h"
-#include "gnome-cmd-main-win-funcs.h"
-#include "gnome-cmd-string-dialog.h"
-#include "widget-factory.h"
+#include "gnome-cmd-file-selector.h"
+#include "gnome-cmd-main-win.h"
 #include "utils.h"
-#include "support.h"
 
 
-static GnomeDialogClass *parent_class = NULL;
+static GnomeCmdDialogClass *parent_class = NULL;
 
 
 struct _GnomeCmdFtpDialogPrivate {
-	GnomeCmdFtpServer *selected_server;
+	GnomeCmdConFtp *selected_server;
 	
 	GtkWidget         *server_list;
 	GtkWidget         *anonymous_pw_entry;
@@ -47,28 +43,35 @@ struct _GnomeCmdFtpDialogPrivate {
 	The main ftp dialog
 ******************************************************/
 
+static GList *
+get_ftp_connections ()
+{
+	return gnome_cmd_con_list_get_all_ftp (gnome_cmd_data_get_con_list ());
+}
+
+
 static void
 load_servers (GnomeCmdFtpDialog *dialog)
 {
-	GList *tmp = gnome_cmd_data_get_ftp_servers ();
+	GList *tmp = get_ftp_connections ();
 	GtkCList *server_list = GTK_CLIST (dialog->priv->server_list);
 
 	gtk_clist_clear (server_list);
 
 	if (tmp)
-		dialog->priv->selected_server = (GnomeCmdFtpServer*)tmp->data;
+		dialog->priv->selected_server = GNOME_CMD_CON_FTP (tmp->data);
 	
 	while (tmp)
 	{
-		GnomeCmdFtpServer *server = (GnomeCmdFtpServer*)tmp->data;
+		GnomeCmdConFtp *server = GNOME_CMD_CON_FTP (tmp->data);
 		if (server)
 		{
 			int row;
 			gchar *text[4];
 			
 			text[0] = NULL;
-			text[1] = (gchar*)gnome_cmd_ftp_server_get_alias (server);
-			text[2] = (gchar*)gnome_cmd_ftp_server_get_host_name (server);
+			text[1] = (gchar*)gnome_cmd_con_ftp_get_alias (server);
+			text[2] = (gchar*)gnome_cmd_con_ftp_get_host_name (server);
 			text[3] = NULL;
 			row = gtk_clist_append (server_list, text);
 			if (!row)
@@ -88,39 +91,35 @@ load_servers (GnomeCmdFtpDialog *dialog)
 }
 
 
+static gboolean
+do_connect_real (GnomeCmdConFtp *server)
+{
+	GnomeCmdCon *con;
+	GnomeCmdFileSelector *fs;
+	
+	fs = gnome_cmd_main_win_get_active_fs (main_win);
+	con = GNOME_CMD_CON (server);
+
+	gnome_cmd_file_selector_set_connection (
+		fs, GNOME_CMD_CON (server), NULL);
+//		gnome_cmd_dir_new (con, gnome_cmd_con_create_path (con, "/")));
+
+	return FALSE;
+}
+
+
 static void
 do_connect (GtkWidget *dialog,
-			GnomeCmdFtpServer *server,
+			GnomeCmdConFtp *server,
 			const gchar *password)
 {
-	GnomeCmdConnection *con;
-	GnomeCmdFileSelector *fs, *fs2;
+	if (!server) return;
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	gnome_cmd_con_ftp_set_pw (server, password);
 	
-	if (!server)
-		return;
-
-	gtk_widget_hide (GTK_WIDGET (dialog));
-	
-	con = gnome_cmd_connection_new_ftp (
-		gnome_cmd_ftp_server_get_alias (server),
-		gnome_cmd_ftp_server_get_host_name (server),
-		gnome_cmd_ftp_server_get_host_port (server),
-		gnome_cmd_ftp_server_get_user_name (server),
-		password,
-		NULL);
-	con->server = server;
-
-	fs = gnome_cmd_main_win_get_active_fs (main_win);
-	fs2 = gnome_cmd_main_win_get_inactive_fs (main_win);
-
-	gnome_cmd_file_selector_update_connections (fs);
-	gnome_cmd_file_selector_update_connections (fs2);
-	
-	if (gnome_cmd_file_selector_set_connection (fs, con, NULL))
-		gnome_cmd_connection_add (con);
-	else
-		gnome_cmd_connection_free (con);
-
+	g_timeout_add (1, (GtkFunction)do_connect_real, server);
 }
 
 
@@ -141,13 +140,13 @@ static void
 on_connect_btn_clicked (GtkButton         *button,
 						GnomeCmdFtpDialog *ftp_dialog)
 {
-	GnomeCmdFtpServer *server = ftp_dialog->priv->selected_server;
+	GnomeCmdConFtp *server = ftp_dialog->priv->selected_server;
 	
 	if (server)
 	{
-		gchar *anon_pw = gtk_entry_get_text (GTK_ENTRY (ftp_dialog->priv->anonymous_pw_entry));
-		const gchar *uname = gnome_cmd_ftp_server_get_user_name (server);
-		const gchar *pw = gnome_cmd_ftp_server_get_pw (server);
+		const gchar *anon_pw = gtk_entry_get_text (GTK_ENTRY (ftp_dialog->priv->anonymous_pw_entry));
+		const gchar *uname = gnome_cmd_con_ftp_get_user_name (server);
+		const gchar *pw = gnome_cmd_con_ftp_get_pw (server);
 		
 		/* store the anonymous password as the user might have changed it */
 		gnome_cmd_data_set_ftp_anonymous_password (anon_pw);
@@ -162,8 +161,8 @@ on_connect_btn_clicked (GtkButton         *button,
 			
 			labels[0] = g_strdup_printf (
 				_("Enter password for %s@%s"),
-				gnome_cmd_ftp_server_get_user_name (server),
-				gnome_cmd_ftp_server_get_host_name (server));
+				gnome_cmd_con_ftp_get_user_name (server),
+				gnome_cmd_con_ftp_get_host_name (server));
 			
 			dialog = gnome_cmd_string_dialog_new (
 				_("Enter Password"),
@@ -189,12 +188,12 @@ static void
 on_cancel_btn_clicked (GtkButton       *button,
 					   GnomeCmdFtpDialog *dialog)
 {
-	gtk_widget_hide (GTK_WIDGET (dialog));
+	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 
 static gchar*
-update_server_from_strings (GnomeCmdFtpServer *server,
+update_server_from_strings (GnomeCmdConFtp *server,
 							const gchar **values,
 							gboolean with_alias)
 {
@@ -222,11 +221,11 @@ update_server_from_strings (GnomeCmdFtpServer *server,
 		return g_strdup_printf (_("Invalid port number: %s"), port);
 
 	if (with_alias)
-		gnome_cmd_ftp_server_set_alias (server, alias);
-	gnome_cmd_ftp_server_set_host_name (server, host);
-	gnome_cmd_ftp_server_set_user_name (server, user);
-	gnome_cmd_ftp_server_set_host_port (server, iport);
-	gnome_cmd_ftp_server_set_pw (server, pw);
+		gnome_cmd_con_ftp_set_alias (server, alias);
+	gnome_cmd_con_ftp_set_host_name (server, host);
+	gnome_cmd_con_ftp_set_user_name (server, user);
+	gnome_cmd_con_ftp_set_host_port (server, iport);
+	gnome_cmd_con_ftp_set_pw (server, pw);
 		
 	return NULL;
 }
@@ -238,15 +237,16 @@ on_new_ftp_server_dialog_ok (GnomeCmdStringDialog *string_dialog,
 							 GnomeCmdFtpDialog    *ftp_dialog)
 {
 	gchar *error_desc;
-	GnomeCmdFtpServer *server = gnome_cmd_ftp_server_new ();
+	GnomeCmdConFtp *server = gnome_cmd_con_ftp_new ("","",0,"","");
 
 	error_desc = update_server_from_strings (server, values, TRUE);
 	if (error_desc != NULL) {
 		gnome_cmd_string_dialog_set_error_desc (string_dialog, error_desc);
-		gnome_cmd_ftp_server_free (server);
+		gtk_object_unref (GTK_OBJECT (server));
 	}
 	else {
-		gnome_cmd_data_add_ftp_server (server);	
+		gnome_cmd_con_list_add_ftp (gnome_cmd_data_get_con_list (),
+									server);	
 		load_servers (ftp_dialog);
 	}
 
@@ -260,7 +260,7 @@ on_edit_ftp_server_dialog_ok (GnomeCmdStringDialog *string_dialog,
 							  GnomeCmdFtpDialog *ftp_dialog)
 {
 	gchar *error_desc;
-	GnomeCmdFtpServer *server = ftp_dialog->priv->selected_server;	
+	GnomeCmdConFtp *server = ftp_dialog->priv->selected_server;	
 
 	error_desc = update_server_from_strings (server, values, TRUE);
 	if (error_desc != NULL)
@@ -276,7 +276,7 @@ static void
 on_pw_entry_changed (GtkEditable *editable,
 					 GnomeCmdStringDialog *string_dialog)
 {
-	gchar *text = gtk_entry_get_text (GTK_ENTRY (editable));
+	const gchar *text = gtk_entry_get_text (GTK_ENTRY (editable));
 	gtk_widget_set_sensitive (string_dialog->entries[string_dialog->rows-1],
 							  !(text && strcmp (text, "anonymous") == 0));
 }
@@ -338,18 +338,18 @@ on_edit_btn_clicked (GtkButton         *button,
 	const gchar *pw;
 	gchar *port;
 	GtkWidget *dialog;
-	GnomeCmdFtpServer *server = ftp_dialog->priv->selected_server;
+	GnomeCmdConFtp *server = ftp_dialog->priv->selected_server;
 
 	g_return_if_fail (server != NULL);
 	
 	dialog = create_ftp_server_dialog (
 		_("Edit FTP Server"), (GnomeCmdStringDialogCallback)on_edit_ftp_server_dialog_ok, ftp_dialog, TRUE);
 		
-	alias = gnome_cmd_ftp_server_get_alias (server);
-	host  = gnome_cmd_ftp_server_get_host_name (server);
-	user  = gnome_cmd_ftp_server_get_user_name (server);
-	pw    = gnome_cmd_ftp_server_get_pw (server);
-	port  = g_strdup_printf ("%d", gnome_cmd_ftp_server_get_host_port (server));
+	alias = gnome_cmd_con_ftp_get_alias (server);
+	host  = gnome_cmd_con_ftp_get_host_name (server);
+	user  = gnome_cmd_con_ftp_get_user_name (server);
+	pw    = gnome_cmd_con_ftp_get_pw (server);
+	port  = g_strdup_printf ("%d", gnome_cmd_con_ftp_get_host_port (server));
 		
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 0, alias);
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 1, host);
@@ -363,11 +363,12 @@ static void
 on_remove_btn_clicked                  (GtkButton       *button,
                                         GnomeCmdFtpDialog *dialog)
 {
-	GnomeCmdFtpServer *server = dialog->priv->selected_server;
+	GnomeCmdConFtp *server = dialog->priv->selected_server;
 	
     if (server)
 	{
-		gnome_cmd_data_remove_ftp_server (server);
+		gnome_cmd_con_list_remove_ftp (gnome_cmd_data_get_con_list (),
+									   server);
 		load_servers (dialog);
 		dialog->priv->selected_server = NULL;
 	}
@@ -397,7 +398,7 @@ on_server_list_select_row                (GtkCList *clist, gint row, gint column
 	else
 	{
 		dialog->priv->selected_server =
-			(GnomeCmdFtpServer*)gtk_clist_get_row_data (clist, row);
+			GNOME_CMD_CON_FTP (gtk_clist_get_row_data (clist, row));
 	}
 }
 
@@ -422,7 +423,7 @@ on_server_list_row_move                  (GtkCList        *clist,
 										  gpointer         user_data)
 {
     gpointer s;
-	GList *servers = gnome_cmd_data_get_ftp_servers ();
+	GList *servers = get_ftp_connections ();
     
     s = g_list_nth_data (servers, arg1);
 	g_return_if_fail (s != NULL);
@@ -430,7 +431,8 @@ on_server_list_row_move                  (GtkCList        *clist,
     servers = g_list_remove (servers, s);    
     servers = g_list_insert (servers, s, arg2);
 
-	gnome_cmd_data_set_ftp_servers (servers);
+	gnome_cmd_con_list_set_all_ftp (
+		gnome_cmd_data_get_con_list (), servers);
 }
 
 
@@ -441,7 +443,7 @@ on_server_list_scroll_vertical                  (GtkCList          *clist,
 												 GnomeCmdFtpDialog *dialog)
 {
 	dialog->priv->selected_server =
-		(GnomeCmdFtpServer*)gtk_clist_get_row_data (clist, clist->focus_row);
+		GNOME_CMD_CON_FTP (gtk_clist_get_row_data (clist, clist->focus_row));
 	
     gtk_clist_select_row (clist, clist->focus_row, 0);
 }
@@ -482,7 +484,7 @@ class_init (GnomeCmdFtpDialogClass *class)
 	object_class = GTK_OBJECT_CLASS (class);
 	widget_class = GTK_WIDGET_CLASS (class);
 
-	parent_class = gtk_type_class (gnome_dialog_get_type ());
+	parent_class = gtk_type_class (gnome_cmd_dialog_get_type ());
 
 	object_class->destroy = destroy;
 
@@ -492,28 +494,22 @@ class_init (GnomeCmdFtpDialogClass *class)
 static void
 init (GnomeCmdFtpDialog *ftp_dialog)
 {
-	GtkWidget *connect_button;
-	GtkWidget *cancel_button;
-
-	GtkWidget *hbox, *vbox, *cat_box, *table, *cat, *sw, *label, *button, *bbox;
+	GtkWidget *cat_box, *table, *cat, *sw, *label, *button, *bbox;
 	
 	GtkWidget *dialog = GTK_WIDGET (ftp_dialog);
 
 	ftp_dialog->priv = g_new (GnomeCmdFtpDialogPrivate, 1);
 	
-
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("FTP Connect"));
 
-	hbox = create_hbox (dialog, FALSE, 12);
-	gtk_container_add (GTK_CONTAINER (GNOME_DIALOG (dialog)->vbox), hbox);
-	vbox = create_vbox (dialog, FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+	gnome_cmd_dialog_set_transient_for (
+		GNOME_CMD_DIALOG (dialog), GTK_WINDOW (main_win));
 
-	cat_box = create_hbox (dialog, FALSE, 6);
+	cat_box = create_hbox (dialog, FALSE, 12);
 	cat = create_category (dialog, cat_box, _("Servers"));
-	gtk_box_pack_start (GTK_BOX (vbox), cat, TRUE, TRUE, 0);
-
+	gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), cat);
+	
 	sw = create_sw (dialog);
 	gtk_box_pack_start (GTK_BOX (cat_box), sw, TRUE, TRUE, 0);
 
@@ -536,21 +532,21 @@ init (GnomeCmdFtpDialog *ftp_dialog)
 	
 	bbox = create_vbuttonbox (dialog);
 	gtk_box_pack_start (GTK_BOX (cat_box), bbox, FALSE, FALSE, 0);
-	button = create_button (dialog, _("_New..."), on_new_btn_clicked);
+	button = create_button (dialog, _("_New..."), GTK_SIGNAL_FUNC (on_new_btn_clicked));
 	gtk_container_add (GTK_CONTAINER (bbox), button);
 	button = create_named_button (dialog, _("_Edit..."),
-								  "edit_button", on_edit_btn_clicked);
+								  "edit_button", GTK_SIGNAL_FUNC (on_edit_btn_clicked));
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
 	button = create_named_button (dialog, _("_Remove"),
-								  "remove_button", on_remove_btn_clicked);
+								  "remove_button", GTK_SIGNAL_FUNC (on_remove_btn_clicked));
 	gtk_widget_set_sensitive (button, FALSE);
 	gtk_container_add (GTK_CONTAINER (bbox), button);
 
 	
 	table = create_table (dialog, 1, 2);
 	cat = create_category (dialog, table, _("Options"));
-	gtk_box_pack_start (GTK_BOX (vbox), cat, FALSE, TRUE, 0);
+	gnome_cmd_dialog_add_category (GNOME_CMD_DIALOG (dialog), cat);
 
 	label = create_label (dialog, _("Anonymous password:"));
 	table_add (table, label, 0, 0, 0);
@@ -559,23 +555,11 @@ init (GnomeCmdFtpDialog *ftp_dialog)
 		create_entry (dialog, "anonymous_pw_entry", gnome_cmd_data_get_ftp_anonymous_password ());
 	table_add (table, ftp_dialog->priv->anonymous_pw_entry, 1, 0, GTK_FILL);
 
-	
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog),
-											_("Connect"), GNOME_STOCK_PIXMAP_REFRESH);
-	connect_button = g_list_last (GNOME_DIALOG (dialog)->buttons)->data;
-	gtk_widget_ref (connect_button);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "connect_button", connect_button,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (connect_button);
-	GTK_WIDGET_SET_FLAGS (connect_button, GTK_CAN_DEFAULT);
 
-	gnome_dialog_append_button (GNOME_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL);
-	cancel_button = g_list_last (GNOME_DIALOG (dialog)->buttons)->data;
-	gtk_widget_ref (cancel_button);
-	gtk_object_set_data_full (GTK_OBJECT (dialog), "cancel_button", cancel_button,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (cancel_button);
-	GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
+	gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_CANCEL,
+								 GTK_SIGNAL_FUNC (on_cancel_btn_clicked), dialog);
+	gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), _("Connect"),
+								 GTK_SIGNAL_FUNC (on_connect_btn_clicked), dialog);
 
 	gtk_signal_connect (GTK_OBJECT (ftp_dialog->priv->server_list), "select_row",
 						GTK_SIGNAL_FUNC (on_server_list_select_row), dialog);
@@ -587,14 +571,8 @@ init (GnomeCmdFtpDialog *ftp_dialog)
 							  "scroll-vertical",
 							  GTK_SIGNAL_FUNC (on_server_list_scroll_vertical), dialog);
 
-	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
-						GTK_SIGNAL_FUNC (on_cancel_btn_clicked), dialog);
-	gtk_signal_connect (GTK_OBJECT (connect_button), "clicked",
-						GTK_SIGNAL_FUNC (on_connect_btn_clicked), dialog);
-
 
 	gtk_widget_grab_focus (ftp_dialog->priv->server_list);
-	gnome_dialog_set_default (GNOME_DIALOG (ftp_dialog), 0);
 }
 
 
@@ -625,7 +603,7 @@ gnome_cmd_ftp_dialog_get_type         (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		dlg_type = gtk_type_unique (gnome_dialog_get_type (), &dlg_info);
+		dlg_type = gtk_type_unique (gnome_cmd_dialog_get_type (), &dlg_info);
 	}
 	return dlg_type;
 }
@@ -657,13 +635,13 @@ on_quick_connect_ok (GnomeCmdStringDialog *string_dialog,
 					 gpointer not_used)
 {
 	gchar *error_desc;
-	GnomeCmdFtpServer *server = gnome_cmd_data_get_quick_connect_server ();
+	GnomeCmdConFtp *server = gnome_cmd_data_get_quick_connect_server ();
 
 	error_desc = update_server_from_strings (server, values, FALSE);
 	if (error_desc != NULL)
 		gnome_cmd_string_dialog_set_error_desc (string_dialog, error_desc);
 	else
-		do_connect (GTK_WIDGET (string_dialog), server, gnome_cmd_ftp_server_get_pw (server));
+		do_connect (GTK_WIDGET (string_dialog), server, gnome_cmd_con_ftp_get_pw (server));
 
 	return error_desc == NULL;
 }
@@ -674,23 +652,22 @@ show_ftp_quick_connect_dialog (void)
 {
 	gchar *port;
 	GtkWidget *dialog;
-	GnomeCmdFtpServer *server = gnome_cmd_data_get_quick_connect_server ();
+	GnomeCmdConFtp *server = gnome_cmd_data_get_quick_connect_server ();
 
 	dialog = create_ftp_server_dialog (_("FTP Quick Connect"),
 									   (GnomeCmdStringDialogCallback)on_quick_connect_ok,
 									   NULL,
 									   FALSE);
 
-	port = g_strdup_printf ("%d", gnome_cmd_ftp_server_get_host_port (server));
+	port = g_strdup_printf ("%d", gnome_cmd_con_ftp_get_host_port (server));
 
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 0,
-									   gnome_cmd_ftp_server_get_host_name (server));
+									   gnome_cmd_con_ftp_get_host_name (server));
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 1, port);
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 2,
-									   gnome_cmd_ftp_server_get_user_name (server));
+									   gnome_cmd_con_ftp_get_user_name (server));
 	gnome_cmd_string_dialog_set_value (GNOME_CMD_STRING_DIALOG (dialog), 3,
 									   gnome_cmd_data_get_ftp_anonymous_password());
 	
 	g_free (port);
 }
-
